%{
/*
 * sim.l - umlsim language
 *
 * Written 2002 by Werner Almesberger
 */


#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include "cmd.h"
#include "y.tab.h"


static int lineno = 1;

%}

%%

time				return TOK_TIME;
exec				return TOK_EXEC;
end				return TOK_END;

[0-9]+(\.[0-9]*)?		{ unsigned long hi,lo;
				  if (sscanf(yytext,"%lu.%lu",&hi,&lo) != 2)
					lo = 0;
				  yylval.tv.tv_sec = hi;
				  yylval.tv.tv_usec = lo;
				  return TOK_TV; }

s				return TOK_S;
ms				return TOK_MS;
us				return TOK_US;

\"[^\"\n\t]+\"			{ *strrchr(yytext,'"') = 0;
				  yylval.str = strdup(yytext+1);
				  if (!yylval.str) {
					perror("malloc");
					exit(1);
				  }
				  return TOK_STRING; }

\n				lineno++;

[\t ]+				;

.				return *yytext;

%%

void yyerror(const char *s)
{
    fprintf(stderr,"%s at line %d, near \"%s\"\n",s,lineno,yytext);
    exit(1);
}
