%{
/*
 * sim.y - umlsim language
 *
 * Written 2002 by Werner Almesberger
 */


#include <stdlib.h>
#include <sys/time.h>

#include "cmd.h"


COMMAND *commands = NULL;

%}

%union {
    int num;
    const char *str;
    const char **stra;
    struct timeval tv;
    COMMAND *cmd;
};

%token		TOK_TIME TOK_EXEC TOK_END
%token		TOK_S TOK_MS TOK_US
%token	<tv>	TOK_TV
%token	<str>	TOK_STRING

%type	<num>	unit
%type	<cmd>	commands command time exec
%type	<stra>	strings

%%

all:
    commands
	{
	    commands = $1;
	}
    ;

commands:
	{
	    $$ = NULL;
	}
    | command commands
	{
	    $$ = $1;
	    $$->next = $2;
	}
    ;

command:
    time
	{
	    $$ = $1;
	}
    | exec
	{
	    $$ = $1;
	}
    ;

time:
    TOK_TIME TOK_TV unit
	{
	    $$ = alloc_cmd(ct_time);

	    $$->u.time = $2;
	    switch ($3) {
		case 0:
		    break;
		case -3:
//@@@
		case -6:
		    $$->u.time.tv_usec = $2.tv_sec;
		    $$->u.time.tv_sec = 0;
		    break;
	    }
	}
    ;

exec:
    TOK_EXEC strings
	{
	}
    ;


unit:
    TOK_S
	{
	    $$ = 0;
	}
    | TOK_MS
	{
	    $$ = -3;
	}
    | TOK_US
	{
	    $$ = -6;
	}
    ;

strings:
	{
	    $$ = NULL;
	}
    | strings TOK_STRING
	{
	    int n = 0;

	    while ($1 && $1[n]) n++;
	    $$ = realloc($1,(n+1)*sizeof(const char *));
	    $$[n] = $2;
	    $$[n+1] = NULL;
	}
    ;
